﻿using Hotline.Application.Snapshot;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Snapshot;
using Microsoft.AspNetCore.Mvc;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Share.Tools;
using Hotline.Snapshot.Interfaces;

namespace Hotline.Api.Controllers.Snapshot;

/// <summary>
/// 随手拍统计
/// </summary>
public class BiSnapshotController : BaseController
{
    private readonly IBiSnapshotApplication _biSnapshotApplication;
    private readonly IIndustryRepository _industryRepository;

    public BiSnapshotController(IBiSnapshotApplication biSnapshotApplication, IIndustryRepository industryRepository)
    {
        _biSnapshotApplication = biSnapshotApplication;
        _industryRepository = industryRepository;
    }

    /// <summary>
    /// 随手拍统计基础数据
    /// </summary>
    /// <returns></returns>
    [HttpGet("statistics/basedata")]
    public async Task<Dictionary<string, object>> GetSnapshotStatisticsBaseDataAsync()
    {
        return new Dictionary<string, object>()
        {
            { "industry", await _industryRepository.GetDataBaseAsync() }
        };
    }

    /// <summary>
    /// 随手拍统计
    /// </summary>
    /// <returns></returns>
    [HttpGet("statistics")]
    public async Task<SnapshotStatisticsOutDto> GetSnapshotStatisticsAsync([FromQuery] SnapshotStatisticsInDto dto)
        =>  await _biSnapshotApplication.GetSnapshotStatisticsAsync(dto, HttpContext.RequestAborted);

    /// <summary>
    /// 随手拍统计详情集合
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpGet("statistics/details")]
    public async Task<PagedDto<SnapshotStatisticsDetailOutDto>> GetSnapshotStatisticsDetailAsync([FromQuery] SnapshotStatisticsDetailInDto dto)
        => (await _biSnapshotApplication.GetSnapshotStatisticsDetail(dto, HttpContext.RequestAborted).ToPagedListAsync(dto)).ToPaged();
    

}
